﻿import sys
import importlib
from typing import Callable

_hook_modules = {}

def register_hook(module: str, function: str, hooker: Callable):
    if module not in _hook_modules:
        _hook_modules[module] = {}
    _hook_modules[module][function] = hooker

def module_hook(fullname, module):
    for k, _hooker in _hook_modules[fullname].items():
        _orig = module.__dict__[k]
        # maybe can do something else here?
        module.__dict__[k] = _hooker(_orig)

class HookedMetaPathFinder:
    def find_module(self, fullname, path=None):
        if fullname in _hook_modules:
            return HookedLoader()

class HookedLoader:
    def load_module(self, fullname):
        if fullname in sys.modules:
            return sys.modules[fullname]

        # need to pop here cuz importlib.import_module will use meta_path
        __hooked_finder = sys.meta_path.pop(0)
        module = importlib.import_module(fullname)

        # rua!
        module_hook(fullname, module)

        # put it back
        sys.meta_path.insert(0, __hooked_finder)
        return module

sys.meta_path.insert(0, HookedMetaPathFinder())

